/*-
 * Copyright (c) 2006 Joseph Koshy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: rawfile.c 223 2008-08-10 15:40:06Z jkoshy $
 */

#include <libelf.h>
#include <string.h>

#include "elfts.h"
#include "tet_api.h"

/*
 * Test the `elf_rawfile' entry point.
 */

IC_REQUIRES_VERSION_INIT();

/*
 * A NULL `elf *' argument should return the appropriate error,
 * and set the `sz' pointer to zero.
 */
void
tcNull_tpNonNull(void)
{
	size_t sz;

	TP_CHECK_INITIALIZATION();

	tet_infoline("assertion: elf_rawfile(NULL,...) returns an error, "
	    "and sets the size pointer to zero.");

	sz = -1;
	if (elf_rawfile(NULL, &sz) != NULL ||
	    elf_errno() != ELF_E_ARGUMENT ||
	    sz != 0)
		tet_result(TET_FAIL);
	else
		tet_result(TET_PASS);
}

static char ar_file[] = "!<arch>\n"
	"t/              1151656346  1001  0     100644  5         `\n"
	"Test\n";

void
tcValidAr_tp1(void)
{
	char *p;
	Elf *e;
	size_t sz;

	TP_CHECK_INITIALIZATION();

	tet_infoline("assertion: elf_rawfile(E,...) descriptor with a valid"
	    " descriptor `E' to an ar(1) archive succeeds and returns the correct"
	    " values.");

	TS_OPEN_MEMORY(e, ar_file);

	if ((p = elf_rawfile(e, &sz)) == NULL ||
	    sz != sizeof(ar_file) ||
	    memcmp(p, ar_file, sizeof(ar_file)))
		tet_result(TET_FAIL);
	else
		tet_result(TET_PASS);

	(void) elf_end(e);
}

static char elf_file[] = "\177ELF\001\001\001	\001\000\000\000\000"
	"\000\000\000\001\000\003\000\001\000\000\000\357\276\255\336"
	"\000\000\000\000\000\000\000\000\003\000\000\0004\000 \000"
	"\000\000(\000\000\000\000\000";

void
tcValidElf_tp1(void)
{
	char *p;
	Elf *e;
	size_t sz;

	TP_CHECK_INITIALIZATION();

	tet_infoline("assertion: elf_rawfile(E,...) descriptor with a valid"
	    " descriptor `E' to an ELF object succeeds and returns the correct"
	    " values.");

	TS_OPEN_MEMORY(e, elf_file);

	if ((p = elf_rawfile(e, &sz)) == NULL ||
	    sz != sizeof(elf_file) ||
	    memcmp(p, elf_file, sizeof(elf_file)))
		tet_result(TET_FAIL);
	else
		tet_result(TET_PASS);

	(void) elf_end(e);
}

void
tcValid_tpNull(void)
{
	char *p;
	Elf *e;

	TP_CHECK_INITIALIZATION();

	tet_infoline("assertion: elf_rawfile(E,NULL) on a valid descriptor "
	    "`E' and NULL sz pointer succeeds and returns the correct "
	    "value.");

	TS_OPEN_MEMORY(e, elf_file);

	if ((p = elf_rawfile(e, NULL)) == NULL ||
	    memcmp(p, elf_file, sizeof(elf_file)))
		tet_result(TET_FAIL);
	else
		tet_result(TET_PASS);

	(void) elf_end(e);
}

/*
 * Todo:
 *
 * Test elf_rawfile() on an ELF object embedded inside an `ar' archive.
 */
